#!/bin/sh

if [ -z "${FILE_TO_SCAN}" ] || [ -z "${PKG_NAME}" ] || [ -z "${PKG_VER}" ]; then
    printf "\e[0;31m%-6s\e[m\n" "** ERROR: file to scan or package info is missing. Aborting.."
    exit 1
fi

. ./google_chat.sh

PRJ_NAME=""

case "${PKG_NAME}" in
*"CB_App_Ctrl"*)
    PRJ_NAME="appc"
    ;;
*"CB_EDR"*)
    PRJ_NAME="edr"
    ;;
*"CB_Cloud"*)
    PRJ_NAME="cbc"
    ;;
*)
    PRJ_NAME="carbonblack"
    ;;
esac

if [ -z "${RL_ARTIFACTORY_FOLDER_URL}" ]; then
    RL_ARTIFACTORY_FOLDER_URL="N/A"
fi

FILE_NAME=""
JOB_URL=""
RL_STORE="/root"
RL_PRJ_DIR=".rl-secure/projects/${PRJ_NAME}"
RL_PKG_DIR="${RL_PRJ_DIR}/packages"
RL_REPORT_DIR="${RL_PKG_DIR}/${PKG_NAME}/versions/${PKG_VER}/reports"
HTML_REPORT="${RL_REPORT_DIR}/rl-html*/"
JSON_REPORT="${RL_REPORT_DIR}/*.rl.json"
LOG_FILE="${REP_PATH_CLI}/rl_${PKG_NAME}_${PKG_VER}_status.out"
RL_PRJ_PURL="pkg:rl/${PRJ_NAME}"
RL_PKG_PURL="${RL_PRJ_PURL}/${PKG_NAME}@${PKG_VER}"
rl_license_output=$(rl-secure license status --no-color)
rl_scan_limit=$(echo "$rl_license_output" | awk '/CLI/ {print $6}')
rl_scan_usage=$(echo "$rl_license_output" | awk '/CLI/ {print $3}')
rl_scan_usage_unit=$(echo "$rl_license_output" | awk '/CLI/ {print $4}')

case "${rl_scan_usage_unit}" in
"GB") ;;
"MB")
    rl_scan_usage=$(echo "scale=2; $rl_scan_usage / 1024.0" | bc)
    ;;
"KB")
    rl_scan_usage=$(echo "scale=2; $rl_scan_usage / 1024.0 / 1024.0" | bc)
    ;;
"B")
    rl_scan_usage=$(echo "scale=2; $rl_scan_usage / 1024.0 / 1024.0 / 1024.0" | bc)
    ;;
*)
    printf "\e[0;31m%-6s\e[m\n" "** ERROR: ReversingLabs won't run! The file size unit could not been determined."
    exit 1
    ;;
esac

if [ "${BUILD_URL}" ]; then
    # we're in jenkins
    JOB_URL="${BUILD_URL}console"
    RL_STORE="${REP_PATH_CLI}"
elif [ "${CI_PROJECT_URL}" ] && [ "${CI_JOB_ID}" ]; then
    # we're in gitlab
    JOB_URL="${CI_PROJECT_URL}/-/jobs/${CI_JOB_ID}"
else
    JOB_URL="UNAVAILABLE"
fi

if echo "${FILE_TO_SCAN}" | grep -qi "https://"; then
    FILE_NAME=$(echo "${FILE_TO_SCAN}" | sed 's/.*\/\([^\/?]*\).*/\1/')

    curl --user ${ARTIFACTORY_USER}:${ARTIFACTORY_PASSWORD} -l ${FILE_TO_SCAN} -o ${FILE_NAME}   
    
elif [ -f "${REP_PATH_CLI}/${FILE_TO_SCAN}" ]; then
    FILE_NAME="${REP_PATH_CLI}/${FILE_TO_SCAN}"
else
    printf "\e[0;31m%-6s\e[m\n" "** ERROR: File not found or invalid. Aborting .."
    exit 1
fi

file_size_b=$(stat -c "%s" "${FILE_NAME}")
file_size_gb=$(echo "scale=2; $file_size_b / 1024.0 / 1024.0 / 1024.0" | bc)

printf "\e[0;33m%-6s\e[m\n" "** INFO: File name set to: ${FILE_NAME}"
printf "\e[0;33m%-6s\e[m\n" "** INFO: File size is: ${file_size_gb}GB"

estimated_new_data_usage=$(echo "scale=2; $rl_scan_usage + $file_size_gb" | bc)

if [ "$(echo "${estimated_new_data_usage} >= ${rl_scan_limit}" | bc -l)" -eq 1 ]; then
    google_chat_channel="PSEC"

    printf "\e[0;31m%-6s\e[m\n" "** ERROR: ReversingLabs won't run! The total scans size will exceed the maximum scan usage allowance of 10GB."
    printf "\e[0;33m%-6s\e[m\n" "** INFO: Estimated new scan usage: ${estimated_new_data_usage}GB"
    notify "*ReversingLabs won't run! The total scans size will exceed the maximum scan usage allowance of 10Gb!* \
        \n- Package name: ${PKG_NAME} \
        \n- Package Version: ${PKG_VER} \
        \n- Package Size: ${file_size_gb}GB \
        \n- Estimated New Scan Usage: ${estimated_new_data_usage}GB \
        \n- Job URL: ${JOB_URL}" \
        $google_chat_channel
    exit 1
else
    printf "\e[0;33m%-6s\e[m\n" "** INFO: There is scanning quota available. Proceeding with the scan .."
    printf "\e[0;33m%-6s\e[m\n" "** INFO: Current scan usage: ${rl_scan_usage}GB"
    printf "\e[0;33m%-6s\e[m\n" "** INFO: Estimated new scan usage: ${estimated_new_data_usage}GB"
fi

{
    printf "\e[0;33m%-6s\e[m\n" "** INFO: Initiating the ReversingLabs scanning activities for ${PKG_NAME}@${PKG_VER} .."

    if [ ! -d "${RL_STORE}/.rl-secure" ]; then
        printf "\e[0;33m%-6s\e[m\n" "** INFO: Initiating the package store .."
        rl-secure init --rl-store="${RL_STORE}"
    fi

    if [ ! -d "${RL_STORE}/${RL_PRJ_DIR}" ]; then
        printf "\e[0;33m%-6s\e[m\n" "** INFO: Creating the root project .."
        rl-secure create --purl="${RL_PRJ_PURL}" --rl-store="${RL_STORE}"
    fi

    rl-secure --version

    rl-secure scan \
        --file-path "${FILE_NAME}" \
        --purl "${RL_PKG_PURL}" \
        --rl-store "${RL_STORE}" \
        --replace \
        --keep-reference \
        --no-tracking

    printf "\e[0;33m%-6s\e[m\n" "** INFO: Retrieving the scan status .."
    rl-secure inspect \
        --purl "${RL_PKG_PURL}" \
        --rl-store "${RL_STORE}" \
        --show-all \
        --fail-only \
        --no-color |
        tee "${LOG_FILE}"

    RL_REPORT_ARGS="--format rl-html,rl-json --purl ${RL_PKG_PURL} --rl-store ${RL_STORE} --no-tracking"
    _RL_PREV_PKG_VER=""

    printf "\e[0;33m%-6s\e[m\n" "** INFO: Checking for previous scan versions of this package .."

    if [ -z "${RL_PREV_PKG_VER}" ]; then
        NUM_OF_VER=$(find "${RL_STORE}/${RL_PKG_DIR}/${PKG_NAME}/versions/" \
            -maxdepth 1 \
            -mindepth 1 \
            -type d |
            wc -l)

        if [ "${NUM_OF_VER}" -gt 1 ]; then
            _RL_PREV_PKG_VER=$(basename "$(find "${RL_STORE}/${RL_PKG_DIR}/${PKG_NAME}/versions/" \
                -maxdepth 1 \
                -mindepth 1 \
                -type d |
                sort |
                tail -n 2 |
                head -n 1)")

            RL_REPORT_ARGS="${RL_REPORT_ARGS} --diff-with ${_RL_PREV_PKG_VER}"

            printf "\e[0;31m%-6s\e[m\n" "** INFO: More than one version automatically detected for ${PKG_NAME}"
            printf "\e[0;31m%-6s\e[m\n" "** INFO: Performing a diff report against version ${_RL_PREV_PKG_VER}"
        fi
    else
        printf "\e[0;31m%-6s\e[m\n" "** INFO: Previous package version provided for ${PKG_NAME}"
        printf "\e[0;31m%-6s\e[m\n" "** INFO: Performing a diff report against version ${RL_PREV_PKG_VER}"
        RL_REPORT_ARGS="${RL_REPORT_ARGS} --diff-with ${RL_PREV_PKG_VER}"
    fi

    printf "\e[0;33m%-6s\e[m\n" "** INFO: Generating final report .."
    rl-secure report ${RL_REPORT_ARGS}
} || {
    printf "\e[0;31m%-6s\e[m\n" "** ERROR: Failed to execute one or more functions of ReversingLabs! Notifying the Product Security team .."
    google_chat_channel="PSEC"

    notify "*Failed to execute one or more functions of ReversingLabs!* \
        \n- Package name: ${PKG_NAME} \
        \n- Package Version: ${PKG_VER} \
        \n- Package Size: ${file_size_gb}GB \
        \n- Job URL: ${JOB_URL}" \
        $google_chat_channel
    exit 1
}

tar -czf "${REP_PATH_CLI}/${PKG_NAME}_${PKG_VER}.tar.gz" -C ${RL_STORE}/${HTML_REPORT} .

printf "\e[0;33m%-6s\e[m\n" "** INFO: Checking if malware has been found .."

if grep -qi "malware" "${LOG_FILE}"; then
    google_chat_channel="PSEC"

    grep -i "malware" <"${LOG_FILE}" | cut -d "]" -f 3 | grep -v -e "^$" | sed -e 's/^\s*//' | while IFS= read -r m; do
        printf "\e[0;31m%-6s\e[m\n" "** CRITICAL: Malware has been found by ReversingLabs! Finding: ${m}"
        notify "*Malware has been found by ReversingLabs!* \
                \n- Package name: ${PKG_NAME} \
                \n- Package Version: ${PKG_VER} \
                \n- Package Size: ${file_size_gb}GB \
                \n- Finding: ${m} \
                \n- Job URL: ${JOB_URL} \
                \n- Link to Report: ${RL_ARTIFACTORY_FOLDER_URL}" \
            $google_chat_channel
    done
fi

report=$(find "${RL_STORE}/${RL_REPORT_DIR}/" -type f -name "*.rl.json" -exec cat {} \;)
assessments=$(echo "$report" | jq -r '.report.metadata.assessments')
summary=""

for key in $(echo "${assessments}" | jq -r "keys[]"); do
    category=$key
    count=$(echo "$assessments" | jq -r ".\"$key\".count")
    label=$(echo "$assessments" | jq -r ".\"$key\".label")
    status=$(echo "$assessments" | jq -r ".\"$key\".status")

    case "$status" in
    "pass")
        status_icon="\uD83D\uDFE2"
        ;;
    "warning")
        status_icon="\uD83D\uDFE1"
        ;;
    "fail")
        status_icon="\uD83D\uDD34"
        ;;
    *)
        status_icon=''
        ;;
    esac

    summaryLine="${status_icon} ${category}: ${count} - ${label}"
    summary="${summary}${summaryLine}\n"
done

printf "\e[0;33m%-6s\e[m\n" "** INFO: Scan and report activities complete for ${PKG_NAME}@${PKG_VER}"
google_chat_channel="MALWARE"

notify "*ReversingLabs scan and report activities complete!* \
    \n- Package name: ${PKG_NAME} \
    \n- Package Version: ${PKG_VER} \
    \n- Package Size: ${file_size_gb}GB \
    \n- Job URL: ${JOB_URL} \
    \n- Link to Report: ${RL_ARTIFACTORY_FOLDER_URL} \
    \n- Summary: \
    \n${summary}" \
    $google_chat_channel

chmod -R 777 "${RL_STORE}"
cp ${RL_STORE}/${JSON_REPORT} "${REP_PATH_CLI}/rl_${PKG_NAME}_${PKG_VER}_report.json"
